#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

// Functions called FROM here to Fortran:

PROTOCCALLSFSUB20(NONEQ,noneq,FLOATV,INT,FLOATV,INT,FLOATV,INT,INT,INT,DOUBLEV,
        DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,INTV,FLOATV,
        INTV,INTV)
#define NONEQ(tempr,ntp,tau,n,weight,nzmax,nzpmax,nionp,vr,vl,eig,feqb,feqs,fs, \
              fspec,work,lt,fout,ionel,ionstage) \
           CCALLSFSUB20(NONEQ,noneq,FLOATV,INT,FLOATV,INT,FLOATV,INT,INT,INT,DOUBLEV, \
             DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,DOUBLEV,INTV,FLOATV, \
             INTV,INTV,tempr,ntp,tau,n,weight,nzmax,nzpmax,nionp,vr,vl,eig,feqb,feqs, \
             fs,fspec,work,lt,fout,ionel,ionstage)     


// Functions called from Fortran TO here:
void ionsneqs(float* tmp, int ntmp, float* tau, int n, float* weight, 
                int nzmax, int nionp, float* fout, int* ionel, 
                int* ionstage);
FCALLSCSUB10(ionsneqs,IONSNEQS,ionsneqs,FLOATV,INT,FLOATV,INT,FLOATV,INT,INT,
        FLOATV,INTV,INTV)


void ionsneqs(float* tmp, int ntmp, float* tau, int n, float* weight, 
                int nzmax, int nionp, float* fout, int* ionel, 
                int* ionstage)
{

//
//        Nonequilibrium ionization ionic concentrations for solar
//        abundances, with a distribution of ionization timescales.
//
//        Input:  tmp      - temperatures (K)
//                ntmp     - number of temperatures
//                tau      - ionization timescales  (cm^-3 s)
//                n        - number of ionization timescales
//                weight   - ionization timescale weights
//                nzmax    - max number of ions tabulated for any element
//                nionp    - number of ions
//        Output: fout     - ionic concentrations
//                ionel    - the element for each fout entry
//                ionstage - the ion stage for each fout entry
//                           (ionel+1=fully stripped)
//

   using namespace XSutility;
   
//  Grab memory for the temporary arrays required by NONEQ.

   auto_array_ptr<double> apVr(new double[nzmax*nzmax*ntmp]);
   auto_array_ptr<double> apVl(new double[nzmax*nzmax*ntmp]);
   auto_array_ptr<double> apEig(new double[nzmax*ntmp]);
   auto_array_ptr<double> apFeqb(new double[(nzmax+1)*ntmp]);
   auto_array_ptr<double> apFeqs(new double[nzmax+1]);
   auto_array_ptr<double> apFs(new double[nzmax+1]);
   auto_array_ptr<double> apFspec(new double[nzmax]);
   auto_array_ptr<double> apWork(new double[nzmax]);
   auto_array_ptr<int> apLt(new int[ntmp]);
   
   for (int i=0; i<nionp; ++i)
      fout[i] = 0.0;

   int nzpmax = nzmax+1;      
   NONEQ(tmp, ntmp, tau, n, weight, nzmax, nzpmax, nionp, apVr.get(),
          apVl.get(), apEig.get(), apFeqb.get(), apFeqs.get(), apFs.get(),
          apFspec.get(), apWork.get(), apLt.get(), fout, ionel, ionstage);
}
